using Harmony;
using RimWorld;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Reflection;
using RimWorld.Planet;
using UnityEngine;
using Verse;
using Verse.AI;

namespace StorageTweaks
{
	//public static bool TryFindBestBetterStoreCellFor(Thing t, Pawn carrier, Map map, StoragePriority currentPriority, Faction faction, out IntVec3 foundCell, bool needAccurateResult = true)
	[HarmonyPatch(typeof(StoreUtility))]
	[HarmonyPatch("TryFindBestBetterStoreCellFor")]
	public static class StoreUtility_TryFindBestBetterStoreCellFor_Patch
	{
		
		// public static bool forbidStoragePriorityNotify = false;
		private static Dictionary<SlotGroup, StoragePriority> storagePriorities = new Dictionary<SlotGroup, StoragePriority>();
		
		
		
		private static void stupidSetPriorityChecksWorkaround(SlotGroup storage, StoragePriority priority)
		{
			Traverse.Create(storage.Settings).Field("priorityInt").SetValue(priority);
		}
		
		
		private static StoragePriority applyByThreshold(Thing thing, SlotGroup storage, StorageTweaksData storageTweaksData)
		{
			if(storageTweaksData.haulThreshold < 1f)
			{
				int stored = StorageTweaksUtility.storedOfThing(thing, storage);
				int maxStored = StorageTweaksUtility.maxStoredOfThing(thing, storage, storageTweaksData.storageLimit);
				
				// Log.Message("T : " + thing.stackCount + ":" + thing.PositionHeld);
				// Log.Message("S : " + stored + "/" + maxStored);
				// Log.Message("S2: " + (float) stored / maxStored + "/" + storageTweaksData.haulThreshold);
				
				// string report = "THR: " + thing + ":" + thing.stackCount + thing.PositionHeld + " ";
				// report = report + stored + "/" + maxStored + " " + (float) stored / maxStored + "/" + storageTweaksData.haulThreshold;
				// Log.Message(report);
				
				if(((float) stored / maxStored) >= storageTweaksData.haulThreshold)
				{
					// Log.Message("1: " + thing.stackCount + "/" + Math.Min(thing.def.stackLimit, maxStored));
					// if(thing.stackCount < thing.def.stackLimit || (stored + thing.def.stackLimit) > maxStored)
					if(thing.stackCount < Math.Min(thing.def.stackLimit, maxStored) ||
						maxStored - stored < Math.Min(thing.def.stackLimit, maxStored)
					)
					{
						// Log.Message("2: " + thing.stackCount + ":" + thing.def.stackLimit);
						// Log.Message("2: " + (stored + thing.def.stackLimit) + ":" + maxStored);
						if(!StorageTweaksUtility.isStorageCell(storage, thing.PositionHeld))
						{
							return StoragePriority.Unstored;
						}
						
					}
				}
			}

			return StoragePriority.Normal; //did not change
		}
		
		
		private static StoragePriority applyByStorageLimit(Thing thing, SlotGroup storage, StorageTweaksData storageTweaksData)
		{
			if(storageTweaksData.storageLimit != 1f)
			{
				int amountToAccept = StorageTweaksUtility.amountToAccept(thing, storage, storageTweaksData.storageLimit);
				// Log.Message("ATA: " + amountToAccept + " " + storage.parent);

				if(amountToAccept == 0)
				{
					return StoragePriority.Unstored;
				}
			}

			return StoragePriority.Normal; //did not change
		}
		
		
		private static StoragePriority applyByLocal(Thing thing, SlotGroup storage, StorageTweaksData storageTweaksData)
		{
			if(storageTweaksData.localStorage > 0f)
			{
				if(StorageTweaksUtility.withinDistance(StorageTweaksUtility.storageCenter(storage), thing.PositionHeld, storageTweaksData.localStorage))
				{
					if(thing.GetSlotGroup() == null) {
						StorageTweaksUtility.updateLocalStorageTick(storage.parent, 5);
						return StoragePriority.Important;
					}
				}
			}

			return StoragePriority.Normal; //did not change
		}
		
		
		private static StoragePriority updateStoragePriorities(Thing thing, Map map)
		{
			StoragePriority resultPriority = StoragePriority.Normal;
			
			List<SlotGroup> storagesList = new List<SlotGroup>(map.haulDestinationManager.AllGroupsListForReading);
			
			// Log.Message("T: " + thing + "(" + thing.stackCount + ") " + thing.GetSlotGroup() + " " + carrier);
			
			for(int i = 0; i < storagesList.Count; i++)
			{
				// Log.Message("S: " + (i + 1) + "/" + storagesList.Count + " : " + storagesList[i]);
				
				if(!storagesList[i].Settings.AllowedToAccept(thing))
				{
					continue;
				}
				
				// Log.Message("T: " + thing + thing.stackCount);
				
				StorageTweaksData storageTweaksData = StorageTweaks.registry.data(storagesList[i].parent as ISlotGroupParent);
				StoragePriority priorityToSet = StoragePriority.Normal;
				
				if(StorageTweaksUtility.thingAlreadyInStorage(thing, storagesList[i]))
				{
					if(storageTweaksData.lastStoredTick > 0)
					{
						if(storageTweaksData.lastStoredTick > Find.TickManager.TicksGame)
						{
							priorityToSet = StoragePriority.Important;
							resultPriority = StoragePriority.Important;
						}
						else
						{
							storageTweaksData.lastStoredTick = 0;
						}
					}
				}
				else
				{
					priorityToSet = applyByStorageLimit(thing, storagesList[i], storageTweaksData);
					// Log.Message("1: " + priorityToSet);
					
					priorityToSet = (priorityToSet == StoragePriority.Normal)
						? applyByThreshold(thing, storagesList[i], storageTweaksData)
						: priorityToSet;
					// Log.Message("2: " + priorityToSet);
					
					priorityToSet = (priorityToSet == StoragePriority.Normal)
						? applyByLocal(thing, storagesList[i], storageTweaksData)
						: priorityToSet;
					// Log.Message("3: " + priorityToSet);
				}

				if(priorityToSet != StoragePriority.Normal)
				{
					storagePriorities.Add(storagesList[i], storagesList[i].Settings.Priority);
					// storagesList[i].Settings.Priority = priorityToSet;
					// Log.Message("P: " + priorityToSet);
					stupidSetPriorityChecksWorkaround(storagesList[i], priorityToSet);
				}
			}
			
			return resultPriority;
		}
		
		private static void restoreStoragePriorities(Map map)
		{
			foreach(KeyValuePair<SlotGroup, StoragePriority> entry in storagePriorities)
			{
				// entry.Key.Settings.Priority = entry.Value;
				// Log.Message("EK: " + entry.Key.parent + ":" + entry.Value);
				stupidSetPriorityChecksWorkaround(entry.Key, entry.Value);
			}
			if(storagePriorities.Count > 0)
			{
				map.haulDestinationManager.Notify_HaulDestinationChangedPriority();
			}

			storagePriorities.Clear();
		}
		
		
		
		[HarmonyPrefix]
		public static bool TryFindBestBetterStoreCellFor_Prefix(ref bool __result, Thing t, Map map, ref StoragePriority currentPriority, ref IntVec3 foundCell)
		{
			// Log.Message("===============");
			StoragePriority priority = updateStoragePriorities(t, map);
			// Log.Message("Y: " + priority);
			if(priority != StoragePriority.Normal)
			{
				currentPriority = priority;
			}
			map.haulDestinationManager.Notify_HaulDestinationChangedPriority();
			
			return true;
		}
		
		
		
		[HarmonyPostfix]
		public static void TryFindBestBetterStoreCellFor_Postfix(ref bool __result, Thing t, Map map, ref IntVec3 foundCell)
		{		
			restoreStoragePriorities(map);
			// Log.Message("=======END========" + __result);
		}
	}
	
	
	
	//private static void TryFindBestBetterStoreCellForWorker(Thing t, Pawn carrier, Map map, Faction faction, SlotGroup slotGroup, bool needAccurateResult, ref IntVec3 closestSlot, ref float closestDistSquared, ref StoragePriority foundPriority)
	// [HarmonyPatch(typeof(StoreUtility))]
	// [HarmonyPatch("TryFindBestBetterStoreCellForWorker")]
	// public static class StoreUtility_TryFindBestBetterStoreCellForWorker_Patch
	// {
		// [HarmonyPrefix]
		// public static bool TryFindBestBetterStoreCellForWorker_Prefix(SlotGroup slotGroup)
		// {
			// Log.Message("2: " + slotGroup + slotGroup.Settings.Priority);
			// return true;
		// }
	// }
}