using Harmony;
using RimWorld;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Reflection;
using RimWorld.Planet;
using UnityEngine;
using Verse;
using Verse.AI;

namespace StorageTweaks
{
	
	public static class StorageTweaksUtility
	{
		public static void updateLocalStorageTick(ISlotGroupParent storageParent, int ticks)
		{
			StorageTweaksData storageTweaksData = StorageTweaks.registry.data(storageParent);
			
			if(storageTweaksData.localStorage > 0f)
			{
				storageTweaksData.lastStoredTick = Find.TickManager.TicksGame + ticks;
			}
		}
		
		public static int storedOfThing(Thing thing, SlotGroup storage)
		{
			int stored = 0;
			foreach(Thing storedThing in storage.HeldThings)
			{
				if(thing.def == storedThing.def)
				{
					stored = stored + storedThing.stackCount;
				}
			}
			
			return stored;
		}
		
		public static int maxStoredOfThing(Thing thing, SlotGroup storage, float limit)
		{
			return Mathf.CeilToInt(thing.def.stackLimit * limit * storage.CellsList.Count());
		}
		
		public static int amountToAccept(Thing thing, SlotGroup storage, float limit)
		{
			//Log.Message("Z: " + thing);
			
			int stored = storedOfThing(thing, storage);
			int maxStored = maxStoredOfThing(thing, storage, limit);
			return (stored > maxStored) ? 0 : maxStored - stored;
		}
	
		public static bool withinDistance(IntVec3 a, IntVec3 b, float distance)
		{
			int resultDistance = (a - b).LengthHorizontalSquared;
			//Log.Message("Z: " + resultDistance + " : " + distance * distance);
			return ((distance * distance) > (a - b).LengthHorizontalSquared) ? true : false;
		}
		
		public static bool thingAlreadyInStorage(Thing thing, SlotGroup storage)
		{
			return thing.GetSlotGroup() == storage;
		}
		
		public static IntVec3 storageCenter(SlotGroup storage)
		{
			int x = 0;
			int z = 0;
			int cellsCount = storage.CellsList.Count;
			
			for(int i = 0; i < cellsCount; i++)
			{
				x = x + storage.CellsList[i].x;
				z = z + storage.CellsList[i].z;
			}
			
			x = x / cellsCount;
			z = z / cellsCount;
			
			return new IntVec3(x, 1, z);
			//return IntVec3.FromVector3((storage.CellsList[0] + storage.CellsList[storage.CellsList.Count - 1]).ToVector3() * 0.5f);
		}
		
		public static bool isStorageCell(SlotGroup storage, IntVec3 cell)
		{
			return storage.CellsList.Contains(cell);
		}
	}
}