using Harmony;
using RimWorld;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Reflection;
using RimWorld.Planet;
using UnityEngine;
using Verse;
using Verse.AI;

namespace StorageTweaks
{
	
	public class ITab_StorageTweaks : ITab
	{
		
		private Vector2 tabSize;
		
		private float drawLocalStorageRadius;
		
		
		public ITab_StorageTweaks()
		{
			tabSize = new Vector2(300f, 250f);
			this.size = tabSize;
			this.labelKey = "StorageTweaks_Label";
			
			drawLocalStorageRadius = 0f;
		}
		
		protected override void FillTab()
		{
			StorageTweaksData storageTweaksData = StorageTweaks.registry.data(selectedStorage());
			
			Rect position = new Rect(0f, 0f, 300f, 80f * 4).ContractedBy(5f);
			GUI.BeginGroup(position);
			
			Listing_Standard list = new Listing_Standard();
            list.ColumnWidth = position.width - 10f;
            list.Begin(position);

			//Limit
			string labelText = (storageTweaksData.storageLimit == 1f)
				? "ST_SettingTitle_StorageLimit_Unlimited".Translate()
				: (Mathf.FloorToInt(storageTweaksData.storageLimit * 100)).ToString() + "%";
			labelText = "ST_SettingTitle_StorageLimit".Translate() + ": " + labelText;
            storageTweaksData.storageLimit = SliderWithLabel(
                    list,
                    labelText,
                    "ST_SettingDesc_StorageLimit".Translate(),
                    storageTweaksData.storageLimit,
                    0f, 1f,
                    20
            );
			
			//Local
			labelText = (storageTweaksData.localStorage == 0f)
				? "ST_SettingTitle_LocalStorage_NotLocal".Translate()
				: (Mathf.CeilToInt(storageTweaksData.localStorage)).ToString() + " " + "ST_SettingTitle_LocalStorage_Cells".Translate();
			labelText = "ST_SettingTitle_LocalStorage".Translate() + ": " + labelText;
			storageTweaksData.localStorage = SliderWithLabel(
                    list,
                    labelText,
                    "ST_SettingDesc_LocalStorage".Translate(),
                    storageTweaksData.localStorage,
                    0f, 16f,
                    1
            );
			
			//Batch
			labelText = (storageTweaksData.haulThreshold == 0f)
				? "ST_SettingTitle_HaulThreshold_OnlyFullBatch".Translate()
				: ((storageTweaksData.haulThreshold == 1f) 
					? "ST_SettingTitle_HaulThreshold_Any".Translate()
					: (Mathf.FloorToInt(storageTweaksData.haulThreshold * 100)).ToString() + "%");
			labelText = "ST_SettingTitle_HaulThreshold".Translate() + ": " + labelText;
			storageTweaksData.haulThreshold = SliderWithLabel(
                    list,
                    labelText,
                    "ST_SettingDesc_HaulThreshold".Translate(),
                    storageTweaksData.haulThreshold,
                    0f, 1f,
                    20
            );
			drawLocalStorageRadius = storageTweaksData.localStorage - 1;
			
			//Merging delay
			labelText = (storageTweaksData.mergingDelay == 0)
				? "ST_SettingTitle_MergingDelay_NoDelay".Translate()
				: ((storageTweaksData.mergingDelay == 13) 
					? "ST_SettingTitle_MergingDelay_Never".Translate()
					: storageTweaksData.mergingDelay.ToString() + " " + "ST_SettingTitle_MergingDelay_Sec".Translate());
			labelText = "ST_SettingTitle_MergingDelay".Translate() + ": " + labelText;
			storageTweaksData.mergingDelay = Mathf.RoundToInt(SliderWithLabel(
                    list,
                    labelText,
                    "ST_SettingDesc_MergingDelay".Translate(),
                    storageTweaksData.mergingDelay,
                    0f, 13f,
                    1
            ));
			
			list.End();
			GUI.EndGroup();
		}
		
		public override void TabUpdate()
		{
			if(drawLocalStorageRadius > 0)
			{
				GenDraw.DrawRadiusRing(StorageTweaksUtility.storageCenter(selectedStorage().GetSlotGroup()), drawLocalStorageRadius);
				drawLocalStorageRadius = 0;
			}
		}
		
		public ISlotGroupParent selectedStorage()
		{
			return base.SelObject as ISlotGroupParent;
		}
		
		private float SliderWithLabel(Listing_Standard list, string title, string hint, float v, float min, float max, int round = -1)
        {
			float y = list.CurHeight;
            list.Label(
                    title,
                    -1.0f,
                    null
            );
            float result = list.Slider(v, min, max);
            if(round > 0)
            {
                result = Mathf.RoundToInt(result * round) * (1f / round);
            }
			if (hint != null)
			{
				Rect tooltip = list.GetRect(0f);
				tooltip.height = tooltip.y - y;
				tooltip.y = y;
				TooltipHandler.TipRegion(tooltip, hint);
			}

            return result;
        }
		
	}
}